==> spring-boot-ext-1.2.232-sources.jar: cn/com/duibaboot/ext/autoconfigure/cat/CatMybatisPlugin.java <==
package cn.com.duibaboot.ext.autoconfigure.cat;

import java.sql.SQLException;
import java.text.DateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Properties;

import cn.com.duiba.boot.utils.ExceptionUtils;
import cn.com.duibaboot.ext.autoconfigure.core.utils.CatUtils;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.ParameterMapping;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Invocation;
import org.apache.ibatis.plugin.Plugin;
import org.apache.ibatis.plugin.Signature;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.apache.ibatis.type.TypeHandlerRegistry;

import com.dianping.cat.Cat;
import com.dianping.cat.message.Message;
import com.dianping.cat.message.Transaction;

/**
 * 对MyBatis进行拦截，添加Cat监控
 * 
 * @author xugf
 */
@Intercepts({
    @Signature(method = "query", type = Executor.class, args = {  
            MappedStatement.class, Object.class, RowBounds.class,  
            ResultHandler.class }),
    @Signature(method = "update", type = Executor.class, args = { MappedStatement.class, Object.class }) 
})
public class CatMybatisPlugin implements Interceptor{

	@Override
	public Object intercept(Invocation invocation) throws Throwable {
		if(!CatUtils.isCatEnabled()){
			return invocation.proceed();
		}
		MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];
        //得到 类名-方法
        String[] strArr = mappedStatement.getId().split("\\.");
        String classAndMethod = strArr[strArr.length-2] + "." + strArr[strArr.length-1];
//        System.out.println("class_method = "+ class_method);
        //得到sql语句
//        Object parameter = null;
//        if(invocation.getArgs().length > 1){
//            parameter = invocation.getArgs()[1];
//        }
//        BoundSql boundSql = mappedStatement.getBoundSql(parameter);
//        Configuration configuration = mappedStatement.getConfiguration();
//        String sql = showSql(configuration, boundSql);
//        System.out.println("sql = "+ sql);
        
        //获取SQL类型
        SqlCommandType sqlCommandType = mappedStatement.getSqlCommandType();
        String method = sqlCommandType.name().toLowerCase();
        
//        System.out.println("method = "+ method);
        
        Transaction t = Cat.newTransaction("SQL", classAndMethod);
        Cat.logEvent("SQL.Method", method);
//        Cat.logEvent("SQL.Statement", sql);
        
        Object returnObj;

        CatUtils.TransactionExt transactionExt = new CatUtils.TransactionExt();
        transactionExt.setTransactionName(classAndMethod);
        CatUtils.setCurrentTransaction(transactionExt);

        try {
            returnObj = invocation.proceed();

            if(transactionExt.getTransaction() != null){
                transactionExt.getTransaction().setStatus(Message.SUCCESS);
            }

            t.setStatus(Message.SUCCESS);
        } catch(Exception e) {
            String errorMsg = e.getClass().getSimpleName();
            if(transactionExt.getTransaction() != null){
                transactionExt.getTransaction().setStatus(errorMsg);
            }

            t.setStatus(errorMsg);
            Cat.logError(e);
            throw e;
        } finally{
            if(transactionExt.getTransaction() != null){
                transactionExt.getTransaction().complete();
            }
            CatUtils.clearCurrentTransaction();

            t.complete();
        }
        return returnObj;
	}
	
	public String showSql(Configuration configuration, BoundSql boundSql) {
        Object parameterObject = boundSql.getParameterObject();
        List<ParameterMapping> parameterMappings = boundSql.getParameterMappings();
        String sql = boundSql.getSql().replaceAll("[\\s]+", " ");
        if (!parameterMappings.isEmpty() && parameterObject != null) {
            TypeHandlerRegistry typeHandlerRegistry = configuration.getTypeHandlerRegistry();
            if (typeHandlerRegistry.hasTypeHandler(parameterObject.getClass())) {
                sql = sql.replaceFirst("\\?", getParameterValue(parameterObject));
 
            } else {
                MetaObject metaObject = configuration.newMetaObject(parameterObject);
                for (ParameterMapping parameterMapping : parameterMappings) {
                    String propertyName = parameterMapping.getProperty();
                    if (metaObject.hasGetter(propertyName)) {
                        Object obj = metaObject.getValue(propertyName);
                        sql = sql.replaceFirst("\\?", getParameterValue(obj));
                    } else if (boundSql.hasAdditionalParameter(propertyName)) {
                        Object obj = boundSql.getAdditionalParameter(propertyName);
                        sql = sql.replaceFirst("\\?", getParameterValue(obj));
                    }
                }
            }
        }
        return sql;
    }
	
	private String getParameterValue(Object obj) {
        String value = null;
        if (obj instanceof String) {
            value = "'" + obj.toString() + "'";
        } else if (obj instanceof Date) {
            DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.CHINA);
            value = "'" + formatter.format(new Date()) + "'";
        } else {
            if (obj != null) {
                value = obj.toString();
            } else {
                value = "";
            }
        }
        return value;
    }

	@Override
	public Object plugin(Object target) {
		return Plugin.wrap(target, this);
	}

	@Override
	public void setProperties(Properties arg0) {
		// TODO Auto-generated method stub
		
	}

}

==> spring-boot-ext-1.2.282-sources.jar: cn/com/duibaboot/ext/autoconfigure/cat/CatMybatisPlugin.java <==
package cn.com.duibaboot.ext.autoconfigure.cat;

import cn.com.duibaboot.ext.autoconfigure.core.utils.CatUtils;
import com.dianping.cat.Cat;
import com.dianping.cat.message.Message;
import com.dianping.cat.message.Transaction;
import org.apache.ibatis.cache.CacheKey;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.ParameterMapping;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.apache.ibatis.type.TypeHandlerRegistry;

import java.text.DateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Properties;

/**
 * 对MyBatis进行拦截，添加Cat监控
 * 
 * @author xugf
 */
@Intercepts({
    @Signature(method = "query", type = Executor.class, args = { MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class }),
    @Signature(method = "query", type = Executor.class, args = { MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class, CacheKey.class, BoundSql.class}),
    @Signature(method = "update", type = Executor.class, args = { MappedStatement.class, Object.class })
})
public class CatMybatisPlugin implements Interceptor{

	@Override
	public Object intercept(Invocation invocation) throws Throwable {
		if(!CatUtils.isCatEnabled()){
			return invocation.proceed();
		}
		MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];
        //得到 类名-方法
        String[] strArr = mappedStatement.getId().split("\\.");
        String classAndMethod = strArr[strArr.length-2] + "." + strArr[strArr.length-1];
//        System.out.println("class_method = "+ class_method);
        //得到sql语句
//        Object parameter = null;
//        if(invocation.getArgs().length > 1){
//            parameter = invocation.getArgs()[1];
//        }
//        BoundSql boundSql = mappedStatement.getBoundSql(parameter);
//        Configuration configuration = mappedStatement.getConfiguration();
//        String sql = showSql(configuration, boundSql);
//        System.out.println("sql = "+ sql);
        
        //获取SQL类型
        SqlCommandType sqlCommandType = mappedStatement.getSqlCommandType();
        String method = sqlCommandType.name().toLowerCase();
        
//        System.out.println("method = "+ method);
        
        Transaction t = Cat.newTransaction("SQL", classAndMethod);
        Cat.logEvent("SQL.Method", method);
//        Cat.logEvent("SQL.Statement", sql);
        
        Object returnObj;

        CatUtils.TransactionExt transactionExt = new CatUtils.TransactionExt();
        transactionExt.setTransactionName(classAndMethod);
        CatUtils.setCurrentTransaction(transactionExt);

        try {
            returnObj = invocation.proceed();

            if(transactionExt.getTransaction() != null){
                transactionExt.getTransaction().setStatus(Message.SUCCESS);
            }

            t.setStatus(Message.SUCCESS);
        } catch(Exception e) {
            String errorMsg = e.getClass().getSimpleName();
            if(transactionExt.getTransaction() != null){
                transactionExt.getTransaction().setStatus(errorMsg);
            }

            t.setStatus(errorMsg);
            Cat.logError(e);
            throw e;
        } finally{
            if(transactionExt.getTransaction() != null){
                transactionExt.getTransaction().complete();
            }
            CatUtils.clearCurrentTransaction();

            t.complete();
        }
        return returnObj;
	}
	
	public String showSql(Configuration configuration, BoundSql boundSql) {
        Object parameterObject = boundSql.getParameterObject();
        List<ParameterMapping> parameterMappings = boundSql.getParameterMappings();
        String sql = boundSql.getSql().replaceAll("[\\s]+", " ");
        if (!parameterMappings.isEmpty() && parameterObject != null) {
            TypeHandlerRegistry typeHandlerRegistry = configuration.getTypeHandlerRegistry();
            if (typeHandlerRegistry.hasTypeHandler(parameterObject.getClass())) {
                sql = sql.replaceFirst("\\?", getParameterValue(parameterObject));
 
            } else {
                MetaObject metaObject = configuration.newMetaObject(parameterObject);
                for (ParameterMapping parameterMapping : parameterMappings) {
                    String propertyName = parameterMapping.getProperty();
                    if (metaObject.hasGetter(propertyName)) {
                        Object obj = metaObject.getValue(propertyName);
                        sql = sql.replaceFirst("\\?", getParameterValue(obj));
                    } else if (boundSql.hasAdditionalParameter(propertyName)) {
                        Object obj = boundSql.getAdditionalParameter(propertyName);
                        sql = sql.replaceFirst("\\?", getParameterValue(obj));
                    }
                }
            }
        }
        return sql;
    }
	
	private String getParameterValue(Object obj) {
        String value = null;
        if (obj instanceof String) {
            value = "'" + obj.toString() + "'";
        } else if (obj instanceof Date) {
            DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.CHINA);
            value = "'" + formatter.format(new Date()) + "'";
        } else {
            if (obj != null) {
                value = obj.toString();
            } else {
                value = "";
            }
        }
        return value;
    }

	@Override
	public Object plugin(Object target) {
		return Plugin.wrap(target, this);
	}

	@Override
	public void setProperties(Properties arg0) {
		// TODO Auto-generated method stub
		
	}

}
